@@ -1,5 +1,8 @@
 /*
- * Copyright 1989 - 1994, Julianne Frances Haugh
+ * Copyright (c) 1989 - 1994, Julianne Frances Haugh
+ * Copyright (c) 1996 - 1998, Marek Michałkiewicz
+ * Copyright (c) 2003 - 2005, Tomasz Kłoczko
+ * Copyright (c) 2008       , Nicolas François
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -10,21 +13,21 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. Neither the name of Julianne F. Haugh nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
+ * 3. The name of the copyright holders or contributors may not be used to
+ *    endorse or promote products derived from this software without
+ *    specific prior written permission.
  *
- * THIS SOFTWARE IS PROVIDED BY JULIE HAUGH AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL JULIE HAUGH OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+ * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
+ * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include <config.h>
@@ -32,8 +35,8 @@
 /* Newer versions of Linux libc already have shadow support.  */
 #ifndef HAVE_GETSPNAM
 
-#include "rcsid.h"
-RCSID ("$Id: shadow.c,v 1.11 2005/05/25 19:31:50 kloczek Exp $")
+#ident "$Id: shadow.c 1980 2008-04-27 00:40:09Z nekral-guest $"
+
 #include <sys/types.h>
 #include "prototypes.h"
 #include "defines.h"
@@ -135,7 +138,8 @@ static struct spwd *my_sgetspent (const char *string)
 		return 0;
 	strcpy (spwbuf, string);
 
-	if ((cp = strrchr (spwbuf, '\n')))
+	cp = strrchr (spwbuf, '\n');
+	if (NULL != cp)
 		*cp = '\0';
 
 	/*
@@ -178,7 +182,8 @@ static struct spwd *my_sgetspent (const char *string)
 	 * incorrectly formatted number, unless we are using NIS.
 	 */
 
-	if ((spwd.sp_lstchg = strtol (fields[2], &cpp, 10)) == 0 && *cpp) {
+	spwd.sp_lstchg = strtol (fields[2], &cpp, 10);
+	if ((spwd.sp_lstchg == 0) && *cpp) {
 #ifdef	USE_NIS
 		if (!nis_used)
 			return 0;
@@ -194,7 +199,8 @@ static struct spwd *my_sgetspent (const char *string)
 	 * Get the minimum period between password changes.
 	 */
 
-	if ((spwd.sp_min = strtol (fields[3], &cpp, 10)) == 0 && *cpp) {
+	spwd.sp_min = strtol (fields[3], &cpp, 10);
+	if ((spwd.sp_min == 0) && *cpp) {
 #ifdef	USE_NIS
 		if (!nis_used)
 			return 0;
@@ -210,7 +216,8 @@ static struct spwd *my_sgetspent (const char *string)
 	 * Get the maximum number of days a password is valid.
 	 */
 
-	if ((spwd.sp_max = strtol (fields[4], &cpp, 10)) == 0 && *cpp) {
+	spwd.sp_max = strtol (fields[4], &cpp, 10);
+	if ((spwd.sp_max == 0) && *cpp) {
 #ifdef	USE_NIS
 		if (!nis_used)
 			return 0;
@@ -238,7 +245,8 @@ static struct spwd *my_sgetspent (const char *string)
 	 * Get the number of days of password expiry warning.
 	 */
 
-	if ((spwd.sp_warn = strtol (fields[5], &cpp, 10)) == 0 && *cpp) {
+	spwd.sp_warn = strtol (fields[5], &cpp, 10);
+	if ((spwd.sp_warn == 0) && *cpp) {
 #ifdef	USE_NIS
 		if (!nis_used)
 			return 0;
@@ -255,7 +263,8 @@ static struct spwd *my_sgetspent (const char *string)
 	 * disabled.
 	 */
 
-	if ((spwd.sp_inact = strtol (fields[6], &cpp, 10)) == 0 && *cpp) {
+	spwd.sp_inact = strtol (fields[6], &cpp, 10);
+	if ((spwd.sp_inact == 0) && *cpp) {
 #ifdef	USE_NIS
 		if (!nis_used)
 			return 0;
@@ -272,7 +281,8 @@ static struct spwd *my_sgetspent (const char *string)
 	 * set to expire.
 	 */
 
-	if ((spwd.sp_expire = strtol (fields[7], &cpp, 10)) == 0 && *cpp) {
+	spwd.sp_expire = strtol (fields[7], &cpp, 10);
+	if ((spwd.sp_expire == 0) && *cpp) {
 #ifdef	USE_NIS
 		if (!nis_used)
 			return 0;
@@ -289,7 +299,8 @@ static struct spwd *my_sgetspent (const char *string)
 	 * to have anything other than a valid integer in it.
 	 */
 
-	if ((spwd.sp_flag = strtol (fields[8], &cpp, 10)) == 0 && *cpp) {
+	spwd.sp_flag = strtol (fields[8], &cpp, 10);
+	if ((spwd.sp_flag == 0) && *cpp) {
 #ifdef	USE_NIS
 		if (!nis_used)
 			return 0;
@@ -322,7 +333,8 @@ struct spwd *fgetspent (FILE * fp)
 	if (fgets (buf, sizeof buf, fp) != (char *) 0)
 #endif
 	{
-		if ((cp = strchr (buf, '\n')))
+		cp = strchr (buf, '\n');
+		if (NULL != cp)
 			*cp = '\0';
 #ifdef	USE_NIS
 		if (nis_ignore && IS_NISCHAR (buf[0]))
@@ -360,7 +372,8 @@ struct spwd *getspent (void)
 		 * right away if there is none.
 		 */
 
-		if (!(val = fgetspent (shadow)))
+		val = fgetspent (shadow);
+		if (NULL == val)
 			return 0;
 
 		/*
@@ -456,11 +469,13 @@ struct spwd *getspnam (const char *name)
 		if (yp_match (nis_domain, "shadow.byname", name,
 			      strlen (name), &nis_val, &nis_vallen) == 0) {
 
-			if (cp = strchr (nis_val, '\n'))
+			cp = strchr (nis_val, '\n');
+			if (NULL != cp)
 				*cp = '\0';
 
 			nis_state = middle;
-			if ((sp = my_sgetspent (nis_val))) {
+			sp = my_sgetspent (nis_val);
+			if (NULL != sp) {
 				strcpy (save_name, sp->sp_namp);
 				nis_key = save_name;
 				nis_keylen = strlen (save_name);
