@@ -1,31 +1,46 @@
+/*
+ * Copyright (c) 1990 - 1994, Julianne Frances Haugh
+ * Copyright (c) 1996 - 2000, Marek Michałkiewicz
+ * Copyright (c) 2001       , Michał Moskal
+ * Copyright (c) 2005       , Tomasz Kłoczko
+ * Copyright (c) 2007 - 2008, Nicolas François
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. The name of the copyright holders or contributors may not be used to
+ *    endorse or promote products derived from this software without
+ *    specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+ * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
+ * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ */
 
 #include <config.h>
 
-#ifdef SHADOWPWD
+#ident "$Id: shadowio.c 1980 2008-04-27 00:40:09Z nekral-guest $"
 
-#include "rcsid.h"
-RCSID ("$Id: shadowio.c,v 1.14 2005/03/31 05:14:49 kloczek Exp $")
 #include "prototypes.h"
 #include "defines.h"
-#ifdef HAVE_SHADOW_H
-# include <shadow.h>
-#endif
+#include <shadow.h>
 #include <stdio.h>
 #include "commonio.h"
 #include "shadowio.h"
-struct spwd *__spw_dup (const struct spwd *spent)
-{
-	struct spwd *sp;
-
-	if (!(sp = (struct spwd *) malloc (sizeof *sp)))
-		return NULL;
-	*sp = *spent;
-	if (!(sp->sp_namp = strdup (spent->sp_namp)))
-		return NULL;
-	if (!(sp->sp_pwdp = strdup (spent->sp_pwdp)))
-		return NULL;
-	return sp;
-}
 
 static void *shadow_dup (const void *ent)
 {
@@ -69,13 +84,18 @@ static struct commonio_ops shadow_ops = {
 	shadow_parse,
 	shadow_put,
 	fgets,
-	fputs
+	fputs,
+	NULL,			/* open_hook */
+	NULL			/* close_hook */
 };
 
 static struct commonio_db shadow_db = {
 	SHADOW_FILE,		/* filename */
 	&shadow_ops,		/* ops */
 	NULL,			/* fp */
+#ifdef WITH_SELINUX
+	NULL,			/* scontext */
+#endif
 	NULL,			/* head */
 	NULL,			/* tail */
 	NULL,			/* cursor */
@@ -153,8 +173,5 @@ void __spw_del_entry (const struct commonio_entry *ent)
 /* Sort with respect to passwd ordering. */
 int spw_sort ()
 {
-	extern struct commonio_db *__pw_get_db ();
-
 	return commonio_sort_wrt (&shadow_db, __pw_get_db ());
 }
-#endif
