@@ -1,5 +1,8 @@
 /*
- * Copyright 1989 - 1994, Julianne Frances Haugh
+ * Copyright (c) 1989 - 1994, Julianne Frances Haugh
+ * Copyright (c) 1996 - 1999, Marek Michałkiewicz
+ * Copyright (c) 2003 - 2005, Tomasz Kłoczko
+ * Copyright (c) 2007 - 2008, Nicolas François
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -10,29 +13,29 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. Neither the name of Julianne F. Haugh nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
+ * 3. The name of the copyright holders or contributors may not be used to
+ *    endorse or promote products derived from this software without
+ *    specific prior written permission.
  *
- * THIS SOFTWARE IS PROVIDED BY JULIE HAUGH AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL JULIE HAUGH OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+ * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
+ * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include <config.h>
 
 #ifndef USE_PAM
 
-#include "rcsid.h"
-RCSID ("$Id: obscure.c,v 1.12 2005/03/24 04:29:23 kloczek Exp $")
+#ident "$Id: obscure.c 1980 2008-04-27 00:40:09Z nekral-guest $"
+
 
 /*
  * This version of obscure.c contains modifications to support "cracklib"
@@ -47,7 +50,7 @@ RCSID ("$Id: obscure.c,v 1.12 2005/03/24 04:29:23 kloczek Exp $")
 /*
  * can't be a palindrome - like `R A D A R' or `M A D A M'
  */
- /*ARGSUSED*/ static int palindrome (const char *old, const char *new)
+static int palindrome (unused const char *old, const char *new)
 {
 	int i, j;
 
@@ -64,7 +67,7 @@ RCSID ("$Id: obscure.c,v 1.12 2005/03/24 04:29:23 kloczek Exp $")
  * more than half of the characters are different ones.
  */
 
- /*ARGSUSED*/ static int similar (const char *old, const char *new)
+static int similar (const char *old, const char *new)
 {
 	int i, j;
 
@@ -91,7 +94,7 @@ RCSID ("$Id: obscure.c,v 1.12 2005/03/24 04:29:23 kloczek Exp $")
  * a nice mix of characters.
  */
 
- /*ARGSUSED*/ static int simple (const char *old, const char *new)
+static int simple (unused const char *old, const char *new)
 {
 	int digits = 0;
 	int uppers = 0;
@@ -158,7 +161,7 @@ static const char *password_check (const char *old, const char *new,
 #endif
 
 	if (strcmp (new, old) == 0)
-		return "no change";
+		return _("no change");
 
 	newmono = str_lower (xstrdup (new));
 	oldmono = str_lower (xstrdup (old));
@@ -167,19 +170,19 @@ static const char *password_check (const char *old, const char *new,
 	strcat (wrapped, oldmono);
 
 	if (palindrome (oldmono, newmono))
-		msg = "a palindrome";
+		msg = _("a palindrome");
 
 	if (!msg && strcmp (oldmono, newmono) == 0)
-		msg = "case changes only";
+		msg = _("case changes only");
 
 	if (!msg && similar (oldmono, newmono))
-		msg = "too similar";
+		msg = _("too similar");
 
 	if (!msg && simple (old, new))
-		msg = "too simple";
+		msg = _("too simple");
 
 	if (!msg && strstr (wrapped, newmono))
-		msg = "rotated";
+		msg = _("rotated");
 
 #ifdef HAVE_LIBCRACK
 	/*
@@ -210,12 +213,13 @@ static const char *password_check (const char *old, const char *new,
 	int maxlen, oldlen, newlen;
 	char *new1, *old1;
 	const char *msg;
+	char *result;
 
 	oldlen = strlen (old);
 	newlen = strlen (new);
 
 	if (newlen < getdef_num ("PASS_MIN_LEN", 0))
-		return "too short";
+		return _("too short");
 
 	/*
 	 * Remaining checks are optional.
@@ -227,15 +231,27 @@ static const char *password_check (const char *old, const char *new,
 	if (msg)
 		return msg;
 
+	if ((result = getdef_str ("ENCRYPT_METHOD")) == NULL) {
 	/* The traditional crypt() truncates passwords to 8 chars.  It is
 	   possible to circumvent the above checks by choosing an easy
 	   8-char password and adding some random characters to it...
 	   Example: "password$%^&*123".  So check it again, this time
 	   truncated to the maximum length.  Idea from npasswd.  --marekm */
 
-	if (getdef_bool ("MD5_CRYPT_ENAB"))
-		return NULL;	/* unlimited password length */
+		if (getdef_bool ("MD5_CRYPT_ENAB"))
+			return NULL;
 
+	} else {
+
+		if (   !strcmp (result, "MD5")
+#ifdef USE_SHA_CRYPT
+		    || !strcmp (result, "SHA256")
+		    || !strcmp (result, "SHA512")
+#endif
+		    )
+			return NULL;
+
+	}
 	maxlen = getdef_num ("PASS_MAX_LEN", 8);
 	if (oldlen <= maxlen && newlen <= maxlen)
 		return NULL;
@@ -276,4 +292,6 @@ int obscure (const char *old, const char *new, const struct passwd *pwdp)
 	return 1;
 }
 
+#else				/* !USE_PAM */
+extern int errno;		/* warning: ANSI C forbids an empty source file */
 #endif				/* !USE_PAM */
