@@ -1,5 +1,8 @@
 /*
- * Copyright 1989 - 1994, Julianne Frances Haugh
+ * Copyright (c) 1989 - 1994, Julianne Frances Haugh
+ * Copyright (c) 1996 - 2000, Marek Michałkiewicz
+ * Copyright (c) 2001 - 2006, Tomasz Kłoczko
+ * Copyright (c) 2007 - 2008, Nicolas François
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -10,35 +13,32 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. Neither the name of Julianne F. Haugh nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
+ * 3. The name of the copyright holders or contributors may not be used to
+ *    endorse or promote products derived from this software without
+ *    specific prior written permission.
  *
- * THIS SOFTWARE IS PROVIDED BY JULIE HAUGH AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL JULIE HAUGH OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+ * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
+ * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include <config.h>
 
-#ident "$Id: chfn.c,v 1.38 2005/10/19 15:21:07 kloczek Exp $"
+#ident "$Id: chfn.c 1980 2008-04-27 00:40:09Z nekral-guest $"
 
 #include <fcntl.h>
 #include <pwd.h>
 #include <signal.h>
 #include <stdio.h>
 #include <sys/types.h>
-#ifdef HAVE_SHADOW_H
-#include <shadow.h>
-#endif
 #ifdef WITH_SELINUX
 #include <selinux/selinux.h>
 #include <selinux/av_permissions.h>
@@ -63,6 +63,15 @@ static char workph[BUFSIZ];
 static char homeph[BUFSIZ];
 static char slop[BUFSIZ];
 static int amroot;
+/* Flags */
+static int fflg = 0;		/* -f - set full name                */
+static int rflg = 0;		/* -r - set room number              */
+static int wflg = 0;		/* -w - set work phone number        */
+static int hflg = 0;		/* -h - set home phone number        */
+static int oflg = 0;		/* -o - set other information        */
+#ifdef USE_PAM
+static pam_handle_t *pamh = NULL;
+#endif
 
 /*
  * External identifiers
@@ -73,25 +82,39 @@ static void usage (void);
 static int may_change_field (int);
 static void new_fields (void);
 static char *copy_field (char *, char *, char *);
+static void process_flags (int argc, char **argv);
+static void check_perms (const struct passwd *pw);
+static void update_gecos (const char *user, char *gecos);
+static void get_old_fields (const char *gecos);
 
 /*
  * usage - print command line syntax and exit
  */
 static void usage (void)
 {
-	if (amroot)
+	if (amroot) {
 		fprintf (stderr,
-			 _("Usage: %s [-f full_name] [-r room_no] "
-			   "[-w work_ph]\n"
-			   "\t[-h home_ph] [-o other] [user]\n"), Prog);
-	else
+		         _("Usage: %s [-f full_name] [-r room_no] "
+		           "[-w work_ph]\n"
+		           "\t[-h home_ph] [-o other] [user]\n"), Prog);
+	} else {
 		fprintf (stderr,
-			 _("Usage: %s [-f full_name] [-r room_no] "
-			   "[-w work_ph] [-h home_ph]\n"), Prog);
+		         _("Usage: %s [-f full_name] [-r room_no] "
+		           "[-w work_ph] [-h home_ph]\n"), Prog);
+	}
 	exit (E_USAGE);
 }
 
-
+/*
+ * may_change_field - indicate if the user is allowed to change a given field
+ *                    of her gecos information
+ *
+ *	root can change any field.
+ *
+ *	field should be one of 'f', 'r', 'w', 'h'
+ *
+ *	Return 1 if the user can change the field and 0 otherwise.
+ */
 static int may_change_field (int field)
 {
 	const char *cp;
@@ -113,17 +136,23 @@ static int may_change_field (int field)
 	 * "no" is equivalent to "frwh". Only root can change anything
 	 * if the string is empty or not defined at all.
 	 */
-	if (amroot)
+	if (amroot) {
 		return 1;
+	}
+
 	cp = getdef_str ("CHFN_RESTRICT");
-	if (!cp)
+	if (NULL == cp) {
 		cp = "";
-	else if (strcmp (cp, "yes") == 0)
+	} else if (strcmp (cp, "yes") == 0) {
 		cp = "rwh";
-	else if (strcmp (cp, "no") == 0)
+	} else if (strcmp (cp, "no") == 0) {
 		cp = "frwh";
-	if (strchr (cp, field))
+	}
+
+	if (strchr (cp, field) != NULL) {
 		return 1;
+	}
+
 	return 0;
 }
 
@@ -135,30 +164,35 @@ static int may_change_field (int field)
  */
 static void new_fields (void)
 {
-	printf (_("Enter the new value, or press ENTER for the default\n"));
+	puts (_("Enter the new value, or press ENTER for the default"));
 
-	if (may_change_field ('f'))
+	if (may_change_field ('f')) {
 		change_field (fullnm, sizeof fullnm, _("Full Name"));
-	else
+	} else {
 		printf (_("\tFull Name: %s\n"), fullnm);
+	}
 
-	if (may_change_field ('r'))
+	if (may_change_field ('r')) {
 		change_field (roomno, sizeof roomno, _("Room Number"));
-	else
+	} else {
 		printf (_("\tRoom Number: %s\n"), roomno);
+	}
 
-	if (may_change_field ('w'))
+	if (may_change_field ('w')) {
 		change_field (workph, sizeof workph, _("Work Phone"));
-	else
+	} else {
 		printf (_("\tWork Phone: %s\n"), workph);
+	}
 
-	if (may_change_field ('h'))
+	if (may_change_field ('h')) {
 		change_field (homeph, sizeof homeph, _("Home Phone"));
-	else
+	} else {
 		printf (_("\tHome Phone: %s\n"), homeph);
+	}
 
-	if (amroot)
+	if (amroot) {
 		change_field (slop, sizeof slop, _("Other"));
+	}
 }
 
 /*
@@ -175,82 +209,40 @@ static char *copy_field (char *in, char *out, char *extra)
 {
 	char *cp = NULL;
 
-	while (in) {
-		if ((cp = strchr (in, ',')))
+	while (NULL != in) {
+		cp = strchr (in, ',');
+		if (NULL != cp) {
 			*cp++ = '\0';
+		}
 
-		if (!strchr (in, '='))
+		if (strchr (in, '=') == NULL) {
 			break;
+		}
 
-		if (extra) {
-			if (extra[0])
+		if (NULL != extra) {
+			if ('\0' != extra[0]) {
 				strcat (extra, ",");
+			}
 
 			strcat (extra, in);
 		}
 		in = cp;
 	}
-	if (in && out)
+	if ((NULL != in) && (NULL != out)) {
 		strcpy (out, in);
+	}
 
 	return cp;
 }
 
 /*
- * chfn - change a user's password file information
+ * process_flags - parse the command line options
  *
- *	This command controls the GECOS field information in the password
- *	file entry.
- *
- *	The valid options are
- *
- *	-f	full name
- *	-r	room number
- *	-w	work phone number
- *	-h	home phone number
- *	-o	other information (*)
- *
- *	(*) requires root permission to execute.
+ *	It will not return if an error is encountered.
  */
-int main (int argc, char **argv)
+static void process_flags (int argc, char **argv)
 {
-	char *cp;		/* temporary character pointer       */
-	const struct passwd *pw;	/* password file entry               */
-	struct passwd pwent;	/* modified password file entry      */
-	char old_gecos[BUFSIZ];	/* buffer for old GECOS fields       */
-	char new_gecos[BUFSIZ];	/* buffer for new GECOS fields       */
 	int flag;		/* flag currently being processed    */
-	int fflg = 0;		/* -f - set full name                */
-	int rflg = 0;		/* -r - set room number              */
-	int wflg = 0;		/* -w - set work phone number        */
-	int hflg = 0;		/* -h - set home phone number        */
-	int oflg = 0;		/* -o - set other information        */
-	char *user;
-
-#ifdef USE_PAM
-	pam_handle_t *pamh = NULL;
-	struct passwd *pampw;
-	int retval;
-#endif
-
-	sanitize_env ();
-	setlocale (LC_ALL, "");
-	bindtextdomain (PACKAGE, LOCALEDIR);
-	textdomain (PACKAGE);
-
-	/*
-	 * This command behaves different for root and non-root
-	 * users.
-	 */
-	amroot = (getuid () == 0);
-
-	/*
-	 * Get the program name. The program name is used as a
-	 * prefix to most error messages.
-	 */
-	Prog = Basename (argv[0]);
-
-	OPENLOG ("chfn");
 
 	/* 
 	 * The remaining arguments will be processed one by one and executed
@@ -264,7 +256,7 @@ int main (int argc, char **argv)
 		case 'f':
 			if (!may_change_field ('f')) {
 				fprintf (stderr,
-					 _("%s: Permission denied.\n"), Prog);
+				         _("%s: Permission denied.\n"), Prog);
 				exit (E_NOPERM);
 			}
 			fflg++;
@@ -273,7 +265,7 @@ int main (int argc, char **argv)
 		case 'h':
 			if (!may_change_field ('h')) {
 				fprintf (stderr,
-					 _("%s: Permission denied.\n"), Prog);
+				         _("%s: Permission denied.\n"), Prog);
 				exit (E_NOPERM);
 			}
 			hflg++;
@@ -282,7 +274,7 @@ int main (int argc, char **argv)
 		case 'r':
 			if (!may_change_field ('r')) {
 				fprintf (stderr,
-					 _("%s: Permission denied.\n"), Prog);
+				         _("%s: Permission denied.\n"), Prog);
 				exit (E_NOPERM);
 			}
 			rflg++;
@@ -291,7 +283,7 @@ int main (int argc, char **argv)
 		case 'o':
 			if (!amroot) {
 				fprintf (stderr,
-					 _("%s: Permission denied.\n"), Prog);
+				         _("%s: Permission denied.\n"), Prog);
 				exit (E_NOPERM);
 			}
 			oflg++;
@@ -300,7 +292,7 @@ int main (int argc, char **argv)
 		case 'w':
 			if (!may_change_field ('w')) {
 				fprintf (stderr,
-					 _("%s: Permission denied.\n"), Prog);
+				         _("%s: Permission denied.\n"), Prog);
 				exit (E_NOPERM);
 			}
 			wflg++;
@@ -310,52 +302,23 @@ int main (int argc, char **argv)
 			usage ();
 		}
 	}
+}
 
-	/*
-	 * Get the name of the user to check. It is either the command line
-	 * name, or the name getlogin() returns.
-	 */
-	if (optind < argc) {
-		user = argv[optind];
-		pw = getpwnam (user);
-		if (!pw) {
-			fprintf (stderr, _("%s: unknown user %s\n"), Prog,
-				 user);
-			exit (E_NOPERM);
-		}
-	} else {
-		pw = get_my_pwent ();
-		if (!pw) {
-			fprintf (stderr,
-				 _
-				 ("%s: Cannot determine your user name.\n"),
-				 Prog);
-			exit (E_NOPERM);
-		}
-		user = xstrdup (pw->pw_name);
-	}
-
-#ifdef	USE_NIS
-	/*
-	 * Now we make sure this is a LOCAL password entry for this user ...
-	 */
-	if (__ispwNIS ()) {
-		char *nis_domain;
-		char *nis_master;
-
-		fprintf (stderr,
-			 _("%s: cannot change user `%s' on NIS client.\n"),
-			 Prog, user);
-
-		if (!yp_get_default_domain (&nis_domain) &&
-		    !yp_master (nis_domain, "passwd.byname", &nis_master)) {
-			fprintf (stderr,
-				 _
-				 ("%s: `%s' is the NIS master for this client.\n"),
-				 Prog, nis_master);
-		}
-		exit (E_NOPERM);
-	}
+/*
+ * check_perms - check if the caller is allowed to add a group
+ *
+ *	Non-root users are only allowed to change their gecos field.
+ *	(see also may_change_field())
+ *
+ *	Non-root users must be authenticated.
+ *
+ *	It will not return if the user is not allowed.
+ */
+static void check_perms (const struct passwd *pw)
+{
+#ifdef USE_PAM
+	int retval;
+	struct passwd *pampw;
 #endif
 
 	/*
@@ -373,6 +336,7 @@ int main (int argc, char **argv)
 	 * check if the change is allowed by SELinux policy.
 	 */
 	if ((pw->pw_uid != getuid ())
+	    && (is_selinux_enabled () > 0)
 	    && (selinux_check_passwd_access (PASSWD__CHFN) != 0)) {
 		fprintf (stderr, _("%s: Permission denied.\n"), Prog);
 		closelog ();
@@ -387,13 +351,14 @@ int main (int argc, char **argv)
 	 * any changes can be made. Idea from util-linux chfn/chsh. 
 	 * --marekm
 	 */
-	if (!amroot && getdef_bool ("CHFN_AUTH"))
+	if (!amroot && getdef_bool ("CHFN_AUTH")) {
 		passwd_check (pw->pw_name, pw->pw_passwd, "chfn");
+	}
 
 #else				/* !USE_PAM */
 	retval = PAM_SUCCESS;
 
-	pampw = getpwuid (getuid ());
+	pampw = getpwuid (getuid ()); /* local, no need for xgetpwuid */
 	if (pampw == NULL) {
 		retval = PAM_USER_UNKNOWN;
 	}
@@ -421,98 +386,17 @@ int main (int argc, char **argv)
 		exit (E_NOPERM);
 	}
 #endif				/* USE_PAM */
+}
 
-	/*
-	 * Now get the full name. It is the first comma separated field in
-	 * the GECOS field.
-	 */
-	STRFCPY (old_gecos, pw->pw_gecos);
-	cp = copy_field (old_gecos, fflg ? (char *) 0 : fullnm, slop);
-
-	/*
-	 * Now get the room number. It is the next comma separated field,
-	 * if there is indeed one.
-	 */
-	if (cp)
-		cp = copy_field (cp, rflg ? (char *) 0 : roomno, slop);
-
-	/*
-	 * Now get the work phone number. It is the third field.
-	 */
-	if (cp)
-		cp = copy_field (cp, wflg ? (char *) 0 : workph, slop);
-
-	/*
-	 * Now get the home phone number. It is the fourth field.
-	 */
-	if (cp)
-		cp = copy_field (cp, hflg ? (char *) 0 : homeph, slop);
-
-	/*
-	 * Anything left over is "slop".
-	 */
-	if (cp && !oflg) {
-		if (slop[0])
-			strcat (slop, ",");
-
-		strcat (slop, cp);
-	}
-
-	/*
-	 * If none of the fields were changed from the command line, let the
-	 * user interactively change them.
-	 */
-	if (!fflg && !rflg && !wflg && !hflg && !oflg) {
-		printf (_("Changing the user information for %s\n"), user);
-		new_fields ();
-	}
-
-	/*
-	 * Check all of the fields for valid information
-	 */
-	if (valid_field (fullnm, ":,=")) {
-		fprintf (stderr, _("%s: invalid name: \"%s\"\n"), Prog, fullnm);
-		closelog ();
-		exit (E_NOPERM);
-	}
-	if (valid_field (roomno, ":,=")) {
-		fprintf (stderr, _("%s: invalid room number: \"%s\"\n"),
-			 Prog, roomno);
-		closelog ();
-		exit (E_NOPERM);
-	}
-	if (valid_field (workph, ":,=")) {
-		fprintf (stderr, _("%s: invalid work phone: \"%s\"\n"),
-			 Prog, workph);
-		closelog ();
-		exit (E_NOPERM);
-	}
-	if (valid_field (homeph, ":,=")) {
-		fprintf (stderr, _("%s: invalid home phone: \"%s\"\n"),
-			 Prog, homeph);
-		closelog ();
-		exit (E_NOPERM);
-	}
-	if (valid_field (slop, ":")) {
-		fprintf (stderr,
-			 _("%s: \"%s\" contains illegal characters\n"),
-			 Prog, slop);
-		closelog ();
-		exit (E_NOPERM);
-	}
-
-	/*
-	 * Build the new GECOS field by plastering all the pieces together,
-	 * if they will fit ...
-	 */
-	if (strlen (fullnm) + strlen (roomno) + strlen (workph) +
-	    strlen (homeph) + strlen (slop) > (unsigned int) 80) {
-		fprintf (stderr, _("%s: fields too long\n"), Prog);
-		closelog ();
-		exit (E_NOPERM);
-	}
-	snprintf (new_gecos, sizeof new_gecos, "%s,%s,%s,%s%s%s",
-		  fullnm, roomno, workph, homeph, slop[0] ? "," : "", slop);
+/*
+ * update_gecos - update the gecos fields in the password database
+ *
+ *	Commit the user's entry after changing her gecos field.
+ */
+static void update_gecos (const char *user, char *gecos)
+{
+	const struct passwd *pw;	/* The user's password file entry */
+	struct passwd pwent;		/* modified password file entry */
 
 	/*
 	 * Before going any further, raise the ulimit to prevent colliding
@@ -520,8 +404,8 @@ int main (int argc, char **argv)
 	 * against unexpected signals. Any keyboard signals are set to be
 	 * ignored.
 	 */
-	if (setuid (0)) {
-		fprintf (stderr, _("Cannot change ID to root.\n"));
+	if (setuid (0) != 0) {
+		fputs (_("Cannot change ID to root.\n"), stderr);
 		SYSLOG ((LOG_ERR, "can't setuid(0)"));
 		closelog ();
 		exit (E_NOPERM);
@@ -532,16 +416,15 @@ int main (int argc, char **argv)
 	 * The passwd entry is now ready to be committed back to the
 	 * password file. Get a lock on the file and open it.
 	 */
-	if (!pw_lock ()) {
-		fprintf (stderr,
-			 _
-			 ("Cannot lock the password file; try again later.\n"));
+	if (pw_lock () == 0) {
+		fputs (_("Cannot lock the password file; try again later.\n"),
+		       stderr);
 		SYSLOG ((LOG_WARN, "can't lock /etc/passwd"));
 		closelog ();
 		exit (E_NOPERM);
 	}
-	if (!pw_open (O_RDWR)) {
-		fprintf (stderr, _("Cannot open the password file.\n"));
+	if (pw_open (O_RDWR) == 0) {
+		fputs (_("Cannot open the password file.\n"), stderr);
 		pw_unlock ();
 		SYSLOG ((LOG_ERR, "can't open /etc/passwd"));
 		closelog ();
@@ -555,10 +438,10 @@ int main (int argc, char **argv)
 	 * AUTOSHADOW (or SHADOW_COMPAT in libc).  --marekm
 	 */
 	pw = pw_locate (user);
-	if (!pw) {
+	if (NULL == pw) {
 		pw_unlock ();
 		fprintf (stderr,
-			 _("%s: %s not found in /etc/passwd\n"), Prog, user);
+		         _("%s: %s not found in /etc/passwd\n"), Prog, user);
 		exit (E_NOPERM);
 	}
 
@@ -567,14 +450,14 @@ int main (int argc, char **argv)
 	 * fields remain unchanged.
 	 */
 	pwent = *pw;
-	pwent.pw_gecos = new_gecos;
+	pwent.pw_gecos = gecos;
 
 	/*
 	 * Update the passwd file entry. If there is a DBM file, update that
 	 * entry as well.
 	 */
-	if (!pw_update (&pwent)) {
-		fprintf (stderr, _("Error updating the password entry.\n"));
+	if (pw_update (&pwent) == 0) {
+		fputs (_("Error updating the password entry.\n"), stderr);
 		pw_unlock ();
 		SYSLOG ((LOG_ERR, "error updating passwd entry"));
 		closelog ();
@@ -584,28 +467,258 @@ int main (int argc, char **argv)
 	/*
 	 * Changes have all been made, so commit them and unlock the file.
 	 */
-	if (!pw_close ()) {
-		fprintf (stderr, _("Cannot commit password file changes.\n"));
+	if (pw_close () == 0) {
+		fputs (_("Cannot commit password file changes.\n"), stderr);
 		pw_unlock ();
 		SYSLOG ((LOG_ERR, "can't rewrite /etc/passwd"));
 		closelog ();
 		exit (E_NOPERM);
 	}
-	if (!pw_unlock ()) {
-		fprintf (stderr, _("Cannot unlock the password file.\n"));
+	if (pw_unlock () == 0) {
+		fputs (_("Cannot unlock the password file.\n"), stderr);
 		SYSLOG ((LOG_ERR, "can't unlock /etc/passwd"));
 		closelog ();
 		exit (E_NOPERM);
 	}
+}
+
+/*
+ * get_old_fields - parse the old gecos and use the old value for the fields
+ *                  which are not set on the command line
+ */
+static void get_old_fields (const char *gecos)
+{
+	char *cp;		/* temporary character pointer       */
+	char old_gecos[BUFSIZ];	/* buffer for old GECOS fields       */
+	STRFCPY (old_gecos, gecos);
+
+	/*
+	 * Now get the full name. It is the first comma separated field in
+	 * the GECOS field.
+	 */
+	cp = copy_field (old_gecos, fflg ? (char *) 0 : fullnm, slop);
+
+	/*
+	 * Now get the room number. It is the next comma separated field,
+	 * if there is indeed one.
+	 */
+	if (NULL != cp) {
+		cp = copy_field (cp, rflg ? (char *) 0 : roomno, slop);
+	}
+
+	/*
+	 * Now get the work phone number. It is the third field.
+	 */
+	if (NULL != cp) {
+		cp = copy_field (cp, wflg ? (char *) 0 : workph, slop);
+	}
+
+	/*
+	 * Now get the home phone number. It is the fourth field.
+	 */
+	if (NULL != cp) {
+		cp = copy_field (cp, hflg ? (char *) 0 : homeph, slop);
+	}
+
+	/*
+	 * Anything left over is "slop".
+	 */
+	if ((NULL != cp) && !oflg) {
+		if ('\0' != slop[0]) {
+			strcat (slop, ",");
+		}
+
+		strcat (slop, cp);
+	}
+}
+
+/*
+ * check_fields - check all of the fields for valid information
+ *
+ *	It will not return if a field is not valid.
+ */
+static void check_fields (void)
+{
+	int err;
+	err = valid_field (fullnm, ":,=");
+	if (err > 0) {
+		fprintf (stderr, _("%s: name with non-ASCII characters: '%s'\n"), Prog, fullnm);
+	} else if (err < 0) {
+		fprintf (stderr, _("%s: invalid name: '%s'\n"), Prog, fullnm);
+		closelog ();
+		exit (E_NOPERM);
+	}
+	err = valid_field (roomno, ":,=");
+	if (err > 0) {
+		fprintf (stderr, _("%s: room number with non-ASCII characters: '%s'\n"), Prog, roomno);
+	} else if (err < 0) {
+		fprintf (stderr, _("%s: invalid room number: '%s'\n"),
+		         Prog, roomno);
+		closelog ();
+		exit (E_NOPERM);
+	}
+	if (valid_field (workph, ":,=")) {
+		fprintf (stderr, _("%s: invalid work phone: '%s'\n"),
+		         Prog, workph);
+		closelog ();
+		exit (E_NOPERM);
+	}
+	if (valid_field (homeph, ":,=")) {
+		fprintf (stderr, _("%s: invalid home phone: '%s'\n"),
+		         Prog, homeph);
+		closelog ();
+		exit (E_NOPERM);
+	}
+	err = valid_field (slop, ":");
+	if (err > 0) {
+		fprintf (stderr, _("%s: '%s' contains non-ASCII characters\n"), Prog, slop);
+	} else if (err < 0) {
+		fprintf (stderr,
+		         _("%s: '%s' contains illegal characters\n"),
+		         Prog, slop);
+		closelog ();
+		exit (E_NOPERM);
+	}
+}
+
+/*
+ * chfn - change a user's password file information
+ *
+ *	This command controls the GECOS field information in the password
+ *	file entry.
+ *
+ *	The valid options are
+ *
+ *	-f	full name
+ *	-r	room number
+ *	-w	work phone number
+ *	-h	home phone number
+ *	-o	other information (*)
+ *
+ *	(*) requires root permission to execute.
+ */
+int main (int argc, char **argv)
+{
+	const struct passwd *pw;	/* password file entry               */
+	char new_gecos[BUFSIZ];	/* buffer for new GECOS fields       */
+	char *user;
+
+	sanitize_env ();
+	setlocale (LC_ALL, "");
+	bindtextdomain (PACKAGE, LOCALEDIR);
+	textdomain (PACKAGE);
+
+	/*
+	 * This command behaves different for root and non-root
+	 * users.
+	 */
+	amroot = (getuid () == 0);
+
+	/*
+	 * Get the program name. The program name is used as a
+	 * prefix to most error messages.
+	 */
+	Prog = Basename (argv[0]);
+
+	OPENLOG ("chfn");
+
+	/* parse the command line options */
+	process_flags (argc, argv);
+
+	/*
+	 * Get the name of the user to check. It is either the command line
+	 * name, or the name getlogin() returns.
+	 */
+	if (optind < argc) {
+		user = argv[optind];
+		pw = xgetpwnam (user);
+		if (NULL == pw) {
+			fprintf (stderr, _("%s: unknown user %s\n"), Prog,
+			         user);
+			exit (E_NOPERM);
+		}
+	} else {
+		pw = get_my_pwent ();
+		if (NULL == pw) {
+			fprintf (stderr,
+			         _
+			         ("%s: Cannot determine your user name.\n"),
+			         Prog);
+			exit (E_NOPERM);
+		}
+		user = xstrdup (pw->pw_name);
+	}
+
+#ifdef	USE_NIS
+	/*
+	 * Now we make sure this is a LOCAL password entry for this user ...
+	 */
+	if (__ispwNIS ()) {
+		char *nis_domain;
+		char *nis_master;
+
+		fprintf (stderr,
+		         _("%s: cannot change user '%s' on NIS client.\n"),
+		         Prog, user);
+
+		if (!yp_get_default_domain (&nis_domain) &&
+		    !yp_master (nis_domain, "passwd.byname", &nis_master)) {
+			fprintf (stderr,
+			         _
+			         ("%s: '%s' is the NIS master for this client.\n"),
+			         Prog, nis_master);
+		}
+		exit (E_NOPERM);
+	}
+#endif
+
+	/* Check that the caller is allowed to change the gecos of the
+	 * specified user */
+	check_perms (pw);
+
+	/* If some fields were not set on the command line, load the value from
+	 * the old gecos fields. */
+	get_old_fields (pw->pw_gecos);
+
+	/*
+	 * If none of the fields were changed from the command line, let the
+	 * user interactively change them.
+	 */
+	if (!fflg && !rflg && !wflg && !hflg && !oflg) {
+		printf (_("Changing the user information for %s\n"), user);
+		new_fields ();
+	}
+
+	/*
+	 * Check all of the fields for valid information
+	 */
+	check_fields ();
+
+	/*
+	 * Build the new GECOS field by plastering all the pieces together,
+	 * if they will fit ...
+	 */
+	if ((strlen (fullnm) + strlen (roomno) + strlen (workph) +
+	     strlen (homeph) + strlen (slop)) > (unsigned int) 80) {
+		fprintf (stderr, _("%s: fields too long\n"), Prog);
+		closelog ();
+		exit (E_NOPERM);
+	}
+	snprintf (new_gecos, sizeof new_gecos, "%s,%s,%s,%s%s%s",
+		  fullnm, roomno, workph, homeph, slop[0] ? "," : "", slop);
+
+	/* Rewrite the user's gecos in the passwd file */
+	update_gecos (user, new_gecos);
+
 	SYSLOG ((LOG_INFO, "changed user `%s' information", user));
 
 	nscd_flush_cache ("passwd");
 
 #ifdef USE_PAM
-	if (retval == PAM_SUCCESS)
-		pam_end (pamh, PAM_SUCCESS);
+	pam_end (pamh, PAM_SUCCESS);
 #endif				/* USE_PAM */
 
 	closelog ();
 	exit (E_SUCCESS);
 }
+
