@@ -1,5 +1,8 @@
 /*
- * Copyright 1991 - 1994, Julianne Frances Haugh
+ * Copyright (c) 1991 - 1994, Julianne Frances Haugh
+ * Copyright (c) 1996 - 2000, Marek Michałkiewicz
+ * Copyright (c) 2000 - 2006, Tomasz Kłoczko
+ * Copyright (c) 2007 - 2008, Nicolas François
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -10,29 +13,30 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. Neither the name of Julianne F. Haugh nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
+ * 3. The name of the copyright holders or contributors may not be used to
+ *    endorse or promote products derived from this software without
+ *    specific prior written permission.
  *
- * THIS SOFTWARE IS PROVIDED BY JULIE HAUGH AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL JULIE HAUGH OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+ * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
+ * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include <config.h>
 
-#ident "$Id: groupmod.c,v 1.37 2005/10/19 15:21:07 kloczek Exp $"
+#ident "$Id: groupmod.c 1980 2008-04-27 00:40:09Z nekral-guest $"
 
 #include <ctype.h>
 #include <fcntl.h>
+#include <getopt.h>
 #include <grp.h>
 #include <stdio.h>
 #include <sys/types.h>
@@ -43,6 +47,7 @@
 #include "chkname.h"
 #include "defines.h"
 #include "groupio.h"
+#include "pwio.h"
 #include "nscd.h"
 #include "prototypes.h"
 #ifdef	SHADOWGRP
@@ -63,21 +68,27 @@
  */
 #ifdef	SHADOWGRP
 static int is_shadow_grp;
-#endif
+static int gshadow_locked = 0;
+#endif				/* SHADOWGRP */
+static int group_locked = 0;
+static int passwd_locked = 0;
 static char *group_name;
 static char *group_newname;
+static char *group_passwd;
 static gid_t group_id;
 static gid_t group_newid;
 
 static char *Prog;
 
 static int
- oflg = 0,			/* permit non-unique group ID to be specified with -g */
+    oflg = 0,			/* permit non-unique group ID to be specified with -g */
     gflg = 0,			/* new ID value for the group */
-    nflg = 0;			/* a new name has been specified for the group */
+    nflg = 0,			/* a new name has been specified for the group */
+    pflg = 0;			/* new encrypted password */
 
 /* local function prototypes */
 static void usage (void);
+static void fail_exit (int);
 static void new_grent (struct group *);
 
 #ifdef SHADOWGRP
@@ -89,6 +100,8 @@ static void check_new_name (void);
 static void process_flags (int, char **);
 static void close_files (void);
 static void open_files (void);
+static gid_t get_gid (const char *gidstr);
+static void update_primary_groups (gid_t ogid, gid_t ngid);
 
 /*
  * usage - display usage message and exit
@@ -96,10 +109,34 @@ static void open_files (void);
 
 static void usage (void)
 {
-	fprintf (stderr, _("Usage: groupmod [-g gid [-o]] [-n name] group\n"));
+	fputs (_("Usage: groupmod [options] GROUP\n"
+	         "\n"
+	         "Options:\n"
+	         "  -g, --gid GID                 force use new GID by GROUP\n"
+	         "  -h, --help                    display this help message and exit\n"
+	         "  -n, --new-name NEW_GROUP      force use NEW_GROUP name by GROUP\n"
+	         "  -o, --non-unique              allow using duplicate (non-unique) GID by GROUP\n"
+	         "  -p, --password PASSWORD       use encrypted password for the new password\n"
+	         "\n"), stderr);
 	exit (E_USAGE);
 }
 
+static void fail_exit (int status)
+{
+	if (group_locked) {
+		gr_unlock ();
+	}
+#ifdef	SHADOWGRP
+	if (gshadow_locked) {
+		sgr_unlock ();
+	}
+#endif				/* SHADOWGRP */
+	if (passwd_locked) {
+		pw_unlock();
+	}
+	exit (status);
+}
+
 /*
  * new_grent - updates the values in a group file entry
  *
@@ -113,6 +150,9 @@ static void new_grent (struct group *grent)
 
 	if (gflg)
 		grent->gr_gid = group_newid;
+
+	if (pflg)
+		grent->gr_passwd = group_passwd;
 }
 
 #ifdef	SHADOWGRP
@@ -126,6 +166,9 @@ static void new_sgent (struct sgrp *sgent)
 {
 	if (nflg)
 		sgent->sg_name = xstrdup (group_newname);
+
+	if (pflg)
+		sgent->sg_passwd = group_passwd;
 }
 #endif				/* SHADOWGRP */
 
@@ -156,7 +199,7 @@ static void grp_update (void)
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "modifying group",
 			      group_name, -1, 0);
 #endif
-		exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE);
 	}
 	grp = *ogrp;
 	new_grent (&grp);
@@ -164,9 +207,15 @@ static void grp_update (void)
 	if (is_shadow_grp && (osgrp = sgr_locate (group_name))) {
 		sgrp = *osgrp;
 		new_sgent (&sgrp);
+		if (pflg)
+			grp.gr_passwd = SHADOW_PASSWD_STRING;
 	}
 #endif				/* SHADOWGRP */
 
+	if (gflg) {
+		update_primary_groups (ogrp->gr_gid, group_newid);
+	}
+
 	/*
 	 * Write out the new group file entry.
 	 */
@@ -176,7 +225,7 @@ static void grp_update (void)
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "adding group",
 			      group_name, -1, 0);
 #endif
-		exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE);
 	}
 	if (nflg && !gr_remove (group_name)) {
 		fprintf (stderr, _("%s: error removing group entry\n"), Prog);
@@ -184,7 +233,7 @@ static void grp_update (void)
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "deleting group",
 			      group_name, -1, 0);
 #endif
-		exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE);
 	}
 #ifdef	SHADOWGRP
 
@@ -199,21 +248,21 @@ static void grp_update (void)
 	/*
 	 * Write out the new shadow group entries as well.
 	 */
-	if (!sgr_update (&sgrp)) {
+	if (is_shadow_grp && !sgr_update (&sgrp)) {
 		fprintf (stderr, _("%s: error adding new group entry\n"), Prog);
 #ifdef WITH_AUDIT
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "adding group",
 			      group_name, -1, 0);
 #endif
-		exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE);
 	}
-	if (nflg && !sgr_remove (group_name)) {
+	if (is_shadow_grp && nflg && !sgr_remove (group_name)) {
 		fprintf (stderr, _("%s: error removing group entry\n"), Prog);
 #ifdef WITH_AUDIT
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "deleting group",
 			      group_name, -1, 0);
 #endif
-		exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE);
 	}
       out:
 #endif				/* SHADOWGRP */
@@ -226,9 +275,10 @@ static void grp_update (void)
 		SYSLOG ((LOG_INFO, "change group `%s' to `%s'",
 			 group_name, group_newname));
 
-	if (gflg)
+	if (gflg) {
 		SYSLOG ((LOG_INFO, "change GID for `%s' to %u",
 			 nflg ? group_newname : group_name, group_newid));
+	}
 }
 
 /*
@@ -248,7 +298,7 @@ static void check_new_gid (void)
 		return;
 	}
 
-	if (oflg || !getgrgid (group_newid))
+	if (oflg || !getgrgid (group_newid)) /* local, no need for xgetgrgid */
 		return;
 
 	/*
@@ -259,7 +309,7 @@ static void check_new_gid (void)
 	audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "modify gid", NULL,
 		      group_newid, 0);
 #endif
-	exit (E_GID_IN_USE);
+	fail_exit (E_GID_IN_USE);
 }
 
 /*
@@ -283,6 +333,7 @@ static void check_new_name (void)
 		/*
 		 * If the entry is found, too bad.
 		 */
+		/* local, no need for xgetgrnam */
 		if (getgrnam (group_newname)) {
 			fprintf (stderr,
 				 _("%s: %s is not a unique name\n"), Prog,
@@ -291,7 +342,7 @@ static void check_new_name (void)
 			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
 				      "modifying group", group_name, -1, 0);
 #endif
-			exit (E_NAME_IN_USE);
+			fail_exit (E_NAME_IN_USE);
 		}
 		return;
 	}
@@ -306,7 +357,24 @@ static void check_new_name (void)
 	audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "modifying group", group_name,
 		      -1, 0);
 #endif
-	exit (E_BAD_ARG);
+	fail_exit (E_BAD_ARG);
+}
+
+/*
+ * get_id - validate and get group ID
+ */
+static gid_t get_gid (const char *gidstr)
+{
+	long val;
+	char *errptr;
+
+	val = strtol (gidstr, &errptr, 10);
+	if (*errptr || errno == ERANGE || val < 0) {
+		fprintf (stderr, _("%s: invalid numeric argument '%s'\n"), Prog,
+			 gidstr);
+		fail_exit (E_BAD_ARG);
+	}
+	return val;
 }
 
 /*
@@ -318,37 +386,48 @@ static void check_new_name (void)
  */
 static void process_flags (int argc, char **argv)
 {
-	char *end;
-	int arg;
 
-	while ((arg = getopt (argc, argv, "g:n:o")) != EOF) {
-		switch (arg) {
-		case 'g':
-			gflg++;
-			group_newid = strtoul (optarg, &end, 10);
-			if (*end != '\0') {
-				fprintf (stderr,
-					 _("%s: invalid group %s\n"),
-					 Prog, optarg);
+	{
+		int option_index = 0;
+		int c;
+		static struct option long_options[] = {
+			{"gid", required_argument, NULL, 'g'},
+			{"help", no_argument, NULL, 'h'},
+			{"new-name", required_argument, NULL, 'n'},
+			{"non-unique", no_argument, NULL, 'o'},
+			{"password", required_argument, NULL, 'p'},
+			{NULL, 0, NULL, '\0'}
+		};
+		while ((c =
+			getopt_long (argc, argv, "g:hn:op:",
+				     long_options, &option_index)) != -1) {
+			switch (c) {
+			case 'g':
+				gflg++;
+				group_newid = get_gid (optarg);
 #ifdef WITH_AUDIT
-				audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-					      "modifying group", NULL,
-					      group_newid, 0);
+				audit_logger (AUDIT_USER_CHAUTHTOK,
+					      Prog, "modifying group",
+					      NULL, group_newid, 0);
 #endif
-				exit (E_BAD_ARG);
+				break;
+			case 'n':
+				nflg++;
+				group_newname = optarg;
+				break;
+			case 'o':
+				oflg++;
+				break;
+			case 'p':
+				group_passwd = optarg;
+				pflg++;
+				break;
+			default:
+				usage ();
 			}
-			break;
-		case 'n':
-			nflg++;
-			group_newname = optarg;
-			break;
-		case 'o':
-			oflg++;
-			break;
-		default:
-			usage ();
 		}
 	}
+
 	if (oflg && !gflg)
 		usage ();
 
@@ -368,18 +447,30 @@ static void close_files (void)
 {
 	if (!gr_close ()) {
 		fprintf (stderr, _("%s: cannot rewrite group file\n"), Prog);
-		exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE);
 	}
 	gr_unlock ();
+	group_locked--;
 #ifdef	SHADOWGRP
 	if (is_shadow_grp && !sgr_close ()) {
 		fprintf (stderr,
 			 _("%s: cannot rewrite shadow group file\n"), Prog);
-		exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE);
 	}
-	if (is_shadow_grp)
+	if (is_shadow_grp) {
 		sgr_unlock ();
+		gshadow_locked--;
+	}
 #endif				/* SHADOWGRP */
+	if (gflg) {
+		if (!pw_close ()) {
+			fprintf (stderr,
+			         _("%s: cannot rewrite passwd file\n"), Prog);
+			fail_exit (E_GRP_UPDATE);
+		}
+		pw_unlock();
+		passwd_locked--;
+	}
 }
 
 /*
@@ -391,24 +482,75 @@ static void open_files (void)
 {
 	if (!gr_lock ()) {
 		fprintf (stderr, _("%s: unable to lock group file\n"), Prog);
-		exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE);
 	}
+	group_locked++;
 	if (!gr_open (O_RDWR)) {
 		fprintf (stderr, _("%s: unable to open group file\n"), Prog);
-		exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE);
 	}
 #ifdef	SHADOWGRP
-	if (is_shadow_grp && !sgr_lock ()) {
-		fprintf (stderr,
-			 _("%s: unable to lock shadow group file\n"), Prog);
-		exit (E_GRP_UPDATE);
-	}
-	if (is_shadow_grp && !sgr_open (O_RDWR)) {
-		fprintf (stderr,
-			 _("%s: unable to open shadow group file\n"), Prog);
-		exit (E_GRP_UPDATE);
+	if (is_shadow_grp) {
+		if (!sgr_lock ()) {
+			fprintf (stderr,
+			         _("%s: unable to lock shadow group file\n"),
+			         Prog);
+			fail_exit (E_GRP_UPDATE);
+		}
+		gshadow_locked++;
+		if (!sgr_open (O_RDWR)) {
+			fprintf (stderr,
+			         _("%s: unable to open shadow group file\n"),
+			         Prog);
+			fail_exit (E_GRP_UPDATE);
+		}
 	}
 #endif				/* SHADOWGRP */
+	if (gflg) {
+		if (!pw_lock ()) {
+			fprintf (stderr,
+			         _("%s: unable to lock password file\n"),
+			         Prog);
+			fail_exit (E_GRP_UPDATE);
+		}
+		passwd_locked++;
+		if (!pw_open (O_RDWR)) {
+			fprintf (stderr,
+			         _("%s: unable to open password file\n"),
+			         Prog);
+			fail_exit (E_GRP_UPDATE);
+		}
+	}
+}
+
+void update_primary_groups (gid_t ogid, gid_t ngid)
+{
+	struct passwd *pwd;
+
+	setpwent ();
+	while ((pwd = getpwent ()) != NULL) {
+		if (pwd->pw_gid == ogid) {
+			const struct passwd *lpwd;
+			struct passwd npwd;
+			lpwd = pw_locate (pwd->pw_name);
+			if (NULL == lpwd) {
+				fprintf (stderr,
+				         _("%s: cannot change the primary group of user '%s' from %u to %u, since it is not in the passwd file.\n"),
+				         Prog, pwd->pw_name, ogid, ngid);
+				fail_exit (E_GRP_UPDATE);
+			} else {
+				npwd = *lpwd;
+				npwd.pw_gid = ngid;
+				if (!pw_update (&npwd)) {
+					fprintf (stderr,
+					         _("%s: cannot change the primary group of user '%s' from %u to %u.\n"),
+					         Prog, pwd->pw_name, ogid, ngid);
+					fail_exit (E_GRP_UPDATE);
+				}
+			}
+		}
+	}
+	endpwent ();
 }
 
 /*
@@ -425,11 +567,8 @@ static void open_files (void)
  */
 int main (int argc, char **argv)
 {
-	struct group *grp;
-
 #ifdef USE_PAM
 	pam_handle_t *pamh = NULL;
-	struct passwd *pampw;
 	int retval;
 #endif
 
@@ -453,13 +592,17 @@ int main (int argc, char **argv)
 #ifdef USE_PAM
 	retval = PAM_SUCCESS;
 
-	pampw = getpwuid (getuid ());
-	if (pampw == NULL) {
-		retval = PAM_USER_UNKNOWN;
-	}
+	{
+		struct passwd *pampw;
+		pampw = getpwuid (getuid ()); /* local, no need for xgetpwuid */
+		if (pampw == NULL) {
+			retval = PAM_USER_UNKNOWN;
+		}
 
-	if (retval == PAM_SUCCESS) {
-		retval = pam_start ("groupmod", pampw->pw_name, &conv, &pamh);
+		if (retval == PAM_SUCCESS) {
+			retval = pam_start ("groupmod", pampw->pw_name,
+					    &conv, &pamh);
+		}
 	}
 
 	if (retval == PAM_SUCCESS) {
@@ -478,26 +621,30 @@ int main (int argc, char **argv)
 
 	if (retval != PAM_SUCCESS) {
 		fprintf (stderr, _("%s: PAM authentication failed\n"), Prog);
-		exit (1);
+		fail_exit (1);
 	}
 #endif				/* USE_PAM */
 
 #ifdef SHADOWGRP
 	is_shadow_grp = sgr_file_present ();
 #endif
-	/*
-	 * Start with a quick check to see if the group exists.
-	 */
-	if (!(grp = getgrnam (group_name))) {
-		fprintf (stderr, _("%s: group %s does not exist\n"),
-			 Prog, group_name);
+	{
+		struct group *grp;
+		/*
+		 * Start with a quick check to see if the group exists.
+		 */
+		/* local, no need for xgetgrnam */
+		if (!(grp = getgrnam (group_name))) {
+			fprintf (stderr, _("%s: group %s does not exist\n"),
+				 Prog, group_name);
 #ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "modifying group",
-			      group_name, -1, 0);
+			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+				      "modifying group", group_name, -1, 0);
 #endif
-		exit (E_NOTFOUND);
-	} else
-		group_id = grp->gr_gid;
+			fail_exit (E_NOTFOUND);
+		} else
+			group_id = grp->gr_gid;
+	}
 
 #ifdef WITH_AUDIT
 	/* Set new name/id to original if not specified on command line */
@@ -527,7 +674,7 @@ int main (int argc, char **argv)
 			fprintf (stderr, _("%s: %s is the NIS master\n"),
 				 Prog, nis_master);
 		}
-		exit (E_NOTFOUND);
+		fail_exit (E_NOTFOUND);
 	}
 #endif
 
@@ -544,10 +691,11 @@ int main (int argc, char **argv)
 	open_files ();
 
 	grp_update ();
-	nscd_flush_cache ("group");
 
 	close_files ();
 
+	nscd_flush_cache ("group");
+
 #ifdef USE_PAM
 	if (retval == PAM_SUCCESS)
 		pam_end (pamh, PAM_SUCCESS);
@@ -555,3 +703,4 @@ int main (int argc, char **argv)
 	exit (E_SUCCESS);
 	/* NOT REACHED */
 }
+
